#include <display_syscalls.h>

#define WIDTH 384
#define HEIGHT 216
#define MAX_RAYON_CIRCLE 30

int rvb_color(int rouge, int vert, int bleu)
{
	return ((bleu%32) + ((vert%64) << 6) + ((rouge%32) << 11));
}

int inverse_rvb_color(int rouge, int vert, int bleu)
{
	return (((31-bleu)%32) + (((31-vert)%64) << 6) + (((31-rouge)%32) << 11));
}

void FillVRAM(short color)
{
	int i, color32bits;
	short* VRAM = (short*)0xA8000000;
	color32bits = (color<<16) | color;
	for(i=82944 ; i ; i--)
		*(VRAM++) = color32bits;
}

int key_down(int basic_keycode)
{
	const unsigned short* keyboard_register = (unsigned short*)0xA44B0000;
	int row, col, word, bit;
	row = basic_keycode%10;
	col = basic_keycode/10-1;
	word = row>>1;
	bit = col + ((row&1)<<3);
	return (0 != (keyboard_register[word] & 1<<bit));
}

static unsigned int lastrandom = 0x12345678;

int time_getTicks()
{
	asm("mov.l syscall_adress, r2\n"
	    "mov.l getTicks, r0\n"
	    "jmp @r2\n"
	    "nop\n"
	    "syscall_adress: .long 0x80020070\n"
	    "getTicks: .long 0x02C1");
}

void srand(unsigned int seed)
{
	lastrandom = seed;
}

int rand(void)
{
	lastrandom = 0x41C64E6D*lastrandom + 0x3039;
	return lastrandom >> 16;
}

void Pixel(unsigned short x, unsigned short y, unsigned short color)
{
    __asm__("mov.w	.width,r1\n"    // width -> r1
            "mulu.w	r1,r5\n"        // width * y -> macl    : Get the y pixel offset
            "mov.l	.vram,r1\n"     // vram -> r1           : Sets the VRAM adres in r1
            "sts	macl,r0\n"      // macl -> r0           : Gets macl in r0
            "add	r4,r0\n"        // r0 + x -> r0         : Get the pixel offset in the VRAM
            "shll	r0\n"           // r0 = r0 <<1          : Multiplies by two to get the short offset
            "mov.w	r6,@(r0,r1)\n"  // *r0 = color          : Sets the pixel to the deisred color
            "rts\n"	
            "clrmac\n"
            ".align 1\n"
            ".width:\n"
            "    .short 384\n"
            " 	 .align 2\n" 
            ".vram:\n"
            "    .long -1476395008\n"
            "	 .align 1\n");
}
void Circle(int x0, int y0, int rayon, int couleur)
 {
      int k;
      int x,y,d;
      for (k=0; k <= rayon; k++)
      {
          x = 0;
          y = k;
          d = k - 1;
          while (y >= x) 
          {
              Pixel( x0 + x, y0 + y, couleur);
              Pixel( x0 + y, y0 + x, couleur);
              Pixel( x0 - x, y0 + y, couleur);
              Pixel( x0 - y, y0 + x, couleur);
              Pixel( x0 + x, y0 - y, couleur);
              Pixel( x0 + y, y0 - x, couleur);
              Pixel( x0 - x, y0 - y, couleur);
              Pixel( x0 - y, y0 - x, couleur);
              if (d >= 2*x)
              {          
                  d -=2*x+1;
                  x++;
              }
              else if (d < 2*(k-y))
              {
                  d += 2*y-1;
                  y--;
              }
              else
              {
                  d += 2*(y-x-1);
                  y--;
                  x++;
              }
          }
      }
 }

void main(void) {

	srand(time_getTicks());

	int a, x, y, rayon_cercle = rand()%20+5, nb_cercle = 20;
	int coordonnee_x[nb_cercle], coordonnee_y[nb_cercle], direction_x[nb_cercle], direction_y[nb_cercle];
	int rouge = rand()%32, vert = rand()%32, bleu = rand()%32;
	short color, anti_color;
	
	for(a=0; a<=nb_cercle; a++)
	{
		coordonnee_x[a] = rand()%(WIDTH-2*MAX_RAYON_CIRCLE)+MAX_RAYON_CIRCLE;
		coordonnee_y[a] = rand()%(HEIGHT-2*MAX_RAYON_CIRCLE)+MAX_RAYON_CIRCLE;
		while(direction_x[a] == 0 &&  direction_y[a] == 0)
		{
			direction_x[a] = rand()%7-3;
			direction_y[a] = rand()%7-3;
		}
		
	}
	
	color = rvb_color(rouge, vert, bleu);
	anti_color = inverse_rvb_color(rouge, vert, bleu);
	
	while(!key_down(47))
	{
		for(a=0; a<=nb_cercle; a++)
		{
			x = coordonnee_x[a]+direction_x[a];
			y = coordonnee_y[a]+direction_y[a];
			if(x<0+rayon_cercle || x>384-rayon_cercle)
				direction_x[a]= -direction_x[a];
			if(y<0+rayon_cercle || y>216-rayon_cercle)
				direction_y[a]= -direction_y[a];
			coordonnee_x[a] += direction_x[a];
			coordonnee_y[a] += direction_y[a];
		}
		
		if(key_down(79) && rayon_cercle<MAX_RAYON_CIRCLE)
			rayon_cercle++;
		if(key_down(69) && rayon_cercle>2)
			rayon_cercle--;
		if (key_down(59))
		{
			rouge = rand()%32;
			vert = rand()%32;
			bleu = rand()%32;
			color = rvb_color(rouge, vert, bleu);
			anti_color = inverse_rvb_color(rouge, vert, bleu);
		}
		
		Bdisp_AllClr_VRAM();
		FillVRAM(anti_color);
		for(a=0; a<=nb_cercle; a++)
		{
			Circle(coordonnee_x[a], coordonnee_y[a], rayon_cercle, color);
			Pixel(coordonnee_x[a], coordonnee_y[a], anti_color);
		}		
		Bdisp_PutDisp_DD();
	}
	return;
}

